function [ counts  cells_of_idxx ] = bin_by_thresholds( v, thresholds )
% works like hist, but with thresholds rather than centres, and the second
% value returned is a cell array containing the indices of the elements of
% v that are in each bin. 
% v should be a vector of real or logical values; thresholds should be a
% vector of threshold values (which need not be in order as it is sorted
% within the function as a precaution)

if isempty( thresholds ) 
    error('Empty threshold vector'); 
end

thresholds = sort( cell2mat(thresholds) ); 
n_thresholds = length( thresholds ); 
n_counts = n_thresholds + 1; 

counts = zeros( 1, n_counts ); 
cells_of_idxx = cell( 1, n_counts ); 

idxx = find( v <= thresholds(1) ); 
counts(1) = length( idxx ); 
cells_of_idxx{1} = idxx; 

for cx = 2:n_thresholds
    idxx = find( (  v > thresholds( cx - 1 ) ) & ( v <= thresholds( cx ) )  ) ; 
    counts( cx ) = length( idxx );
    cells_of_idxx{ cx } = idxx; 
end

idxx = find( v > thresholds(n_thresholds) ); 
counts( n_counts ) = length( idxx ); 
cells_of_idxx{n_counts} = idxx; 

end