function [root points plothandle] = construct_mosaic_plot( categories, category_names, thresholds, separations)
% construct_mosaic_plot(Study.data,{'a','b','c','d'},{10,10,10,10});
% construct_mosaic_plot(Study.data(:,1:5),Study.ColumnIDs(:,1:5),num2cell(mean(Study.data(:,1:5))));
% construct_mosaic_plot(Study.data,Study.ColumnIDs,num2cell(mean(Study.data)));
% EXAMPLE LINE
% called as: [root points] = construct_mosaic_plot( categories, category_names, thresholds,
%           separations )
% arguments:
%   categories: n_cats x n_examples matrix, each column is the values for a
%       category in the mosaic plot
%   category_names: (optional - default empty strings ) a cell vector of 
%       length n_cats, containing the names of the categories
%   thresholds:  (optional) cell vector of length n_cats, containing arrays of sorted
%       threshold values to be used in constructing mosaic plot. 
%       Default is that all thresholds are 0.5
%   separations: (optional) a vector of length n_cats containing the
%       separations to use in laying out the mosaic plot
%  
% results: 
%   root:  the root cell of a 'patch-tree', which can be plotted with
%          draw-patches
%   points: a n_data x 2 vector of the x and y coordinates of the points
%          plotted
%   plothandle: the handle of a matlab plot drawn


assert( not(isempty(categories) )); 
n_cats = size( categories, 2 ); 
n_data = size( categories, 1 ); 

if nargin < 4 % we need default separations
    max_separation = 0.15; 
    separations = zeros(n_cats, 1); 
    for x = 1:n_cats
        % separations decrease from 0.15 linearly to zero
        separations(x) = ((n_cats - x)/(n_cats - 1)) * max_separation;
    end
else
    assert( length( separations ) == n_cats ); 
end

if nargin < 3 % we need default thresholds
    thresholds = cell( n_cats, 1 ); 
    for x = 1:n_cats
        thresholds{x} = 0.5; 
    end
else
    assert( length( thresholds ) == n_cats ); 
end

if nargin < 2 % we need default category names 
    category_names = cell(n_cats,1); 
    for x = 1:n_cats
        category_names{x} = ''; 
    end
else
    assert( length( category_names ) == n_cats); 
end

idxx = 1:n_data; 
% future versions of this function might include the facility to define
% idxx differently from additional arguments -- let's keep it simple here
p = initialise_patch( idxx ); 

for level = 1:n_cats 
    p = divide_patch( p, level, categories(:,level), ...
        thresholds{level}, separations(level) ); 
end
%FOR DEBUG RETURN
[pxx pyy] = patches2points( p, n_cats, n_data );
f1 = figure; 
plothandle = plot( pxx, pyy, '.' , 'MarkerSize', 0.001,'Marker','none'); 
% This seems to be the most elegant way to achieve brushable elements



hold on
draw_patches( p, n_cats );
%draw_labels( p, n_cats, category_names,0,thresholds);
draw_labels2(p , category_names,1,thresholds);
axis([0-(n_cats*0.1) 1+(n_cats*0.1) 0-(n_cats*0.1) 1+(n_cats*0.1)])
axis off
hold off
h1 = uicontrol(f1,'Style','PushButton','Units','normalized',...
               'String','Select Points','Callback', 'mosaic_selector();');%,'Position',[1 1 1 1]);
% set up results to return
root = p;
points = [pxx pyy]; 
% plothandle is already assigned. 

end



