function [ vertices faces vertex_colourx vertex_alphax] = ...
    make_vertices_faces_from_neighbours( ...
    vis, neighbours, neighbour_distances, width, mitre, taper, ...
    sort_direction, frac_to_plot, high_alpha, low_alpha )
% cons

[N K] = size( neighbours ); 
n_arrows = K * N; 
n_faces = n_arrows * 3; % 3 triangles per arrow
n_vertices = n_arrows  * 5;

faces = zeros( n_faces, 3 ); 
vertices = zeros( n_vertices, 2 ); 
vertex_colourx = zeros( n_vertices, 1 ); 

vis_width = max( max( vis ) - min( vis ) ) ; 

w0 = width * vis_width; % maximal starting width of an arrow. 
w_quantum = 0.2 * w0; 
d_scale_factor = 0.05 * vis_width; % just a guess -- try computing it from statistics on 

median_neighbour_distance = median( neighbour_distances(:) ); 

rotation90 = [ 0 -1 ; 1 0 ]; 

face_vertex_mat = [ 1 2 4 ; 2 5 4 ; 2 3 5 ] ; 

% construct a list of neighbour indices (paired with k
neighbour_list = [ repmat( (1:N)', [K 1 ]) , neighbours(:), neighbour_distances(:) ]; 

[sorted_hh sorted_hh_idx] = sort( neighbour_distances(:), sort_direction); 
neighbour_list = neighbour_list( sorted_hh_idx, : ); 
neighbour_list = neighbour_list( 1:ceil(K*N*frac_to_plot), :); 

n_arrows_to_plot = size(neighbour_list, 1 ); 
n_faces = n_arrows_to_plot * 3; % 3 triangles per arrow
n_vertices = n_arrows_to_plot  * 5;

faces = zeros( n_faces, 3 ); 
vertices = zeros( n_vertices, 2 ); 
vertex_colourx = zeros( n_vertices, 1 ); 
vertex_alphax = zeros( n_vertices, 1 ); 

% use for loop -- vectorise later
for arrowx = 1:n_arrows_to_plot
        % compute index offsets for vertices and faces
        arrow_offsetx = arrowx - 1; 
        face_offsetx = 3 * arrow_offsetx; 
        vertex_offsetx = 5 * arrow_offsetx; 
        
        % actually the indexes of the vertices that make up each face could
        % all be computed in advance - but in this first implementation the
        % faces matrix is built up arrow by arrow, with three faces being 
        % added for each arrow. 
        faces( (face_offsetx + 1):(face_offsetx + 3), : ) = ...
            face_vertex_mat + vertex_offsetx;
        
        % compute point-to-point vectors in visualisation
        nx = neighbour_list( arrowx, 1 ); 
        avec = vis( nx, : ); 
        bvec = vis( neighbour_list( arrowx, 2), : ); 
        abvec = bvec - avec; 
        d = sqrt( abvec * abvec' ); 
        if d < w0 * 0.000001
            % a and b are essentially the same point in the visualisation,
            % so make up a vector
            abvec = [ 1 0 ];
            uvec = [1 0] ;
        else
            uvec = abvec / d; 
            wvec = uvec * rotation90'; % uvec and wvec are now unit-length row vectors 
        end
        
        % compute width and taper of arrow
        a_width = w0 / ( 1 + sqrt( d_scale_factor * d + w_quantum ) ); % half-width of arrow at start (point a)
        b_width = max( 0 , a_width - taper * w0 ); % half-width of arrow at end (point b)
        
        % compute the arrow vertices
        arrow_vertices = zeros( 5, 2 ); 
        arrow_vertices(1,:) = avec - mitre * a_width * uvec + a_width * wvec;
        arrow_vertices(2,:) = avec - mitre * a_width * uvec; 
        arrow_vertices(3,:) = avec - mitre * a_width * uvec - a_width * wvec; 
        arrow_vertices(4,:) = bvec + mitre * b_width * uvec + b_width * wvec; 
        arrow_vertices(5,:) = bvec + mitre * b_width * uvec - b_width * wvec; 
        
        vertices( (vertex_offsetx + 1):(vertex_offsetx + 5) , : ) = arrow_vertices; 
        
        % colours of starting and finishing ends of arrow may be different
        h = neighbour_list( arrowx, 3 ); 
        start_vertexx = (vertex_offsetx + 1):(vertex_offsetx + 3);
        end_vertexx = (vertex_offsetx + 4):(vertex_offsetx + 5);
        vertex_colourx( start_vertexx ) = ...
            ceil( 64 * (1 - 1 / (1 + exp( - ( h / median_neighbour_distance - 2 ) ) ) )); 
        vertex_colourx( end_vertexx ) = ...
            ceil( 64 * (1 - 1/ (1 + exp( - ( h / median_neighbour_distance - 2 ) ) ) )); 
        vertex_alphax( start_vertexx ) = ceil(high_alpha * 64); 
        vertex_alphax( end_vertexx ) = ceil( low_alpha * 64);
end

vertex_colourx = min(64,  max( 1, vertex_colourx )); 
vertex_alphax = min( 64, max( 1, vertex_alphax )); 
% gets rid of any zeros which will not index a valid colour in the colormap

end