function [ xx yy ] = patches2points( patch_root, level, n_rows )
% returns column vectors of x and y coordinates 
% patch root is the root node of a patch tree
% level is the
% n_rows is the number of rows in the corresponding study table
% xx and yy are column vectors of point coordinates in the multi-histogram


xx = zeros( n_rows, 1 ); 
yy = zeros( n_rows, 1 ); 


% flatten the patch_tree
sub_patches = patches_at_level( patch_root, level ); 
np = length( sub_patches ); 
for px = 1:np
   idxx = sub_patches{px}.idxx; 
   if ~isempty( idxx ) 
       xrange = sub_patches{px}.xx; % [ xmin xmax ]
       yrange = sub_patches{px}.yy; % [ ymin ymax ]
       xside = xrange(2) - xrange(1); 
       yside = yrange(2) - yrange(1); 
       assert( xside > 0 ); 
       assert( yside > 0 ); 
       nx = length( idxx ); 
       if nx > 0 
          pcols = ceil( sqrt( nx * xside / yside ) ); % vertical on screen
          prows = ceil( nx / pcols ); % prows * pcols > nx
          assert( prows * pcols >= nx ); 
          xstep = (xrange(2) - xrange(1)) / pcols; 
          ystep = (yrange(2) - yrange(1)) / prows; 
          for idx = 1:nx
              % idxx( idx ) is the index in the study-table 
              xx( idxx( idx ) ) = xrange(1) + (xstep/2) + xstep * mod( idx-1, pcols ); 
              yy( idxx( idx ) ) = yrange(1) + (ystep/2) + ystep * floor( (idx-1) /pcols ); 
          end
       end
   end
end

end