function [pcprojections pcs pcvals] = pcs_of_z_scores( zscores, plot_or_not )
% This function computes the principal components of a rectangular matrix
%   of z_scores
% Call as: [pcprojections pcs pcvals] = pcs_of_z_scores( zscores )
%   to just calculate the PCs. Plots will appear, and the results shown
%   will be returned 
% Arguments: 
%   zscores is a NxK matrix with each row being z-scores
%     (actually the procedure will work with other types of matrix also)
% Results: 
%   pcprojections:  NxK matrix, each column of which is the projection
%           the data points onto the corresponding PC 
%   pcs : KxK matrix, first column is first principal component of
%       correlation matrix, second column is second PC etc
%   pcvals : K vector of principal component variances: plotting this
%       gives an indication of which PCs are important. 


N = size( zscores, 1 ); 

zmc_zscores = zero_mean_rows( zscores' )';
zcov = (1/N) * ( zmc_zscores' * zmc_zscores );

% imagesc( zcov ); colorbar

invdiagstd = diag( 1 ./ sqrt( diag( zcov) ) ); 

zcorr = invdiagstd * zcov * invdiagstd; 

%figure; imagesc( zcorr ); colorbar

[uspcs uspcvals ] = eig( zcorr ); 

[pcvals sorted_pcvxx] = sort( diag( uspcvals), 'descend' ); 
pcs = uspcs(:, sorted_pcvxx); 

pcprojections = zscores * pcs; 


    % do some plotting 
    
    figure; plot( pcvals, '.','Markersize',20); 
    figure; plot( pcprojections(:,1), pcprojections(:,2), '.' ); 
    K = size( pcprojections, 2 ); 
    figure; plotmatrix( pcprojections(:,1:min(K,5))); 


end

